/****************************************************************************************** 
 *	Chili DirectX Framework Version 16.07.20											  *	
 *	Game.cpp																			  *
 *	Copyright 2016 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "MainWindow.h"
#include "Game.h"
#include "Colors.h"
#include "Game.h"

Game::Game(MainWindow& wind)
	:
	wnd(wind),
	gfx(wind),
	rng(std::random_device()()),
	brd(gfx),
	galaxy(rng),
	mouse(),
	font("Consolas13x24.bmp")
{
	int count = 0;
	Planet* hp = new Planet[GALAXY_SIZE];
	for (int i = 0; i < GALAXY_SIZE; i++)
	{
		if(galaxy.get_sector(i).planet.get_loc()!=EMPTY && galaxy.get_sector(i).planet.is_habitable() == true)
		{
			hp[count] = galaxy.get_sector(i).planet;
			count++;
		}
	}
	if (count < numplayers)
		Game(*wind);
	for (int j = 0; j < numplayers; j++)
	{
		std::uniform_int_distribution<int> dist(0, count - j);
		bool is_colonized = false;
		int roll = 0;
		do
		{
			is_colonized = false;
			roll = dist(rng);
			for (int p = 0; p < numplayers; p++)
			{
				if (player[p].is_colony(hp[roll].get_loc()) == true)
					is_colonized = true;
			}
		} while (is_colonized == true);
		player[j].add_colony(hp[roll]);
		for (int i = 0; i < GALAXY_SIZE; i++)
		{
			if (abs(galaxy.get_sector(i).loc.x - hp[roll].get_loc().x) + abs(galaxy.get_sector(i).loc.y - hp[roll].get_loc().y) <= 2)
			{
				player[j].add_discovered(galaxy.get_sector(i).loc);
				if (galaxy.get_sector(i).planet.get_loc()!=EMPTY)
				{
					player[j].add_discovered_planet(galaxy.get_sector(i).planet);
				}
			}
		}
	}	
	delete[] hp;
}

void Game::Go()
{
	player[0].set_colour(Colors::Cyan);
	player[1].set_colour(Colors::Magenta);
	gfx.BeginFrame();
	UpdateModel();
	ComposeFrame();
	gfx.EndFrame();
}

void Game::UpdateModel()
{
}

void Game::ComposeFrame()
{
	for (int i=0;i<GALAXY_SIZE; i++)
	{
		if (galaxy.get_sector(i).star == true)
		{
			brd.DrawCell(galaxy.get_sector(i).loc, Colors::White);
		}
		if (galaxy.get_sector(i).planet.get_loc()!=EMPTY)
		{
			if (galaxy.get_sector(i).planet.get_habitable() == true)
			{
				brd.DrawCell(galaxy.get_sector(i).loc, Colors::Green);
			}
			else
			{
				brd.DrawCell(galaxy.get_sector(i).loc, Colors::Red);
			}
			for (int p = 0; p < numplayers; p++)
			{
				for (int j = 0; player[p].get_colony(j).get_loc() != EMPTY; j++)
				{
					if (player[p].get_colony(j).get_loc() == galaxy.get_sector(i).loc)
					{
						brd.DrawCell(galaxy.get_sector(i).loc, player[p].get_colour());
					}
				}
			}
		}
	}

	for (int i = 0; i < brd.get_height(); i++)
	{
		gfx.PutPixel(GALAXY_WIDTH * brd.get_dim(), i, Colors::White);
	}

	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = 25 + (i % 3) * 2;
		newloc.y = (i / 3) * 2;
		if (player[0].get_colony(0).get_tile_yield(i) == 0)
		{
			brd.DrawPlanetTile(newloc, Colors::Red);
		}
		else if (player[0].get_colony(0).get_tile_yield(i) ==1)
		{
			brd.DrawPlanetTile(newloc, Colors::Yellow);
		}
		else if (player[0].get_colony(0).get_tile_yield(i) ==2)
		{
			brd.DrawPlanetTile(newloc, Colors::Cyan);
		}
		else if (player[0].get_colony(0).get_tile_yield(i) ==3)
		{
			brd.DrawPlanetTile(newloc, Colors::Green);
		}
	}

	
	std::string text = "Player 1";
	Location loc;
	loc.x = GALAXY_WIDTH * brd.get_dim();
	loc.y = brd.get_height() - font.get_glyphHeight() - 1;
	font.drawText(text, loc, gfx,player[0].get_colour());
}

