#include "Surface.h"
#include <string>
#include <fstream>
#include "ChiliWin.h"
#include <cassert>
#include <stdexcept>

Surface::Surface(const std::string& filename)
{
	std::ifstream file(filename, std::ios::binary);
	if (!file.is_open())
		throw std::runtime_error("File not in current directory.");
	BITMAPFILEHEADER bmFileHeader{};
	file.read(reinterpret_cast<char*>(&bmFileHeader), sizeof(bmFileHeader));
	BITMAPINFOHEADER bmInfoHeader{};
	file.read(reinterpret_cast<char*>(&bmInfoHeader), sizeof(bmInfoHeader));

	assert(bmInfoHeader.biBitCount == 24 || bmInfoHeader.biBitCount == 32);
	assert(bmInfoHeader.biCompression == BI_RGB);

	bool is32 = bmInfoHeader.biBitCount == 32;

	width = bmInfoHeader.biWidth;

	int ystart;
	int yend;
	int dy;
	if (bmInfoHeader.biHeight < 0)
	{
		height = -1 * bmInfoHeader.biHeight;
		ystart = 0;
		yend = height;
		dy = 1;
	}
	else
	{
		height = bmInfoHeader.biHeight;
		ystart = height - 1;
		yend = -1;
		dy = -1;
	}

	pPixels = new Color[width * height];
	file.seekg(bmFileHeader.bfOffBits);
	const int padding = (4 - (width * 3) % 4) % 4;

	for (int y = ystart; y != yend; y+=dy)
	{
		for (int x = 0; x < width; x++)
		{
			const unsigned char b = file.get();
			const unsigned char g = file.get();
			const unsigned char r = file.get();
			pPixels[width * y + x] = { r,g,b };
			if (is32)
			{
				file.seekg(1, std::ios::cur);
			}
		}
		if (!is32)
		{
			file.seekg(padding, std::ios::cur);
		}
	}
}

Surface::Surface(int Width, int Height)
	:
	width(Width),
	height(Height),
	pPixels(new Color[Width*Height])
{
}

void Surface::PutPixel(int x, int y, Color c)
{
	assert(x >= 0);
	assert(x < width);
	assert(y >= 0);
	assert(y < height);
	pPixels[width * y + x] = c;
}

Color Surface::GetPixel(int x, int y) const
{
	assert(x >= 0);
	assert(x < width);
	assert(y >= 0);
	assert(y < height);
	return pPixels[width * y + x];
}
int Surface::GetWidth() const
{
	return width;
}
int Surface::GetHeight() const
{
	return height;
}